% --------------------------------------------------------------
% List Predicates
% Viktor Engelmann
% www.AlgorithMan.de
% --------------------------------------------------------------
% this code is provided under the terms of the GNU General
% Public License v3. See gpl3.txt for details
% --------------------------------------------------------------


algorithman_length([],0) :- !.
algorithman_length([_|R],N) :-
   var(N),
   !,
   algorithman_length(R,N1),
   N is N1 + 1.
algorithman_length([_|R],N) :-
   nonvar(N),
   N > 0,
   N1 is N - 1,
   algorithman_length(R,N1).


%algorithman_member([E|_],E).
%algorithman_member([_|R],E):-
%   algorithman_member(R,E).
algorithman_member(X,L) :- algorithman_append(_, [X|_], L).


algorithman_nthelement([X|_],0,X).
algorithman_nthelement([_|R],N,E):-
   nonvar(N),
   !,
   N1 is N - 1,
   algorithman_nthelement(R,N1,E).
algorithman_nthelement([_|R],N,E):-
   var(N),
   !,
   algorithman_nthelement(R,N1,E),
   N is N1 + 1.
   
   
   
% --------------------------------------------------------------
%                                                   change lists
% --------------------------------------------------------------


algorithman_append([],L,L).
algorithman_append([E|L1] ,L2 ,[E|L3]):-
   algorithman_append(L1,L2,L3).


algorithman_delete([D|R],D,R).
algorithman_delete([E|R],D,[E|L]):-
   algorithman_delete(R,D,L).

   
algorithman_delete_all([],_,[]).
algorithman_delete_all([D|R],D,L):-
   algorithman_delete_all(R,D,L).
algorithman_delete_all([E|R],D,[E|L]):-
   D \= E,
   algorithman_delete_all(R,D,L).


algorithman_del_from_pos([_|R],1,R).
algorithman_del_from_pos([E|R],Pos,[E|L]):-
   Pos > 1,
   P1 is Pos - 1,
   algorithman_del_from_pos(R,P1,L).
   

%algorithman_insert(L,E,[E|L]).
%algorithman_insert([E|L],I,[E|R]):-
%   algorithman_insert(L,I,R).
algorithman_insert(L,E,R):- algorithman_delete(R,E,L).

   
algorithman_insert_position(R,E,1,[E|R]).
algorithman_insert_position([E|R],I,Pos,[E|L]):-
   Pos > 1,
   P1 is Pos - 1,
   algorithman_insert_position(R,I,P1,L).

  
% --------------------------------------------------------------
%                                                     cool stuff
% --------------------------------------------------------------


algorithman_reverse([],[]).
algorithman_reverse([E|R],L):-
   algorithman_reverse(R,Temp),
   algorithman_append(Temp,[E],L).


algorithman_permutation([],[]).
algorithman_permutation([X|L],P):-
   algorithman_permutation(L,LPermutation),
   algorithman_insert(LPermutation, X, P).

   
algorithman_subset([],[]).
algorithman_subset([E|L],[E|T]):-
   algorithman_subset(L,T).
algorithman_subset([_|L],T):-
   algorithman_subset(L,T).

   
algorithman_quicksort_pivot(_, [], [], []):-!.
algorithman_quicksort_pivot(M, [X|R], [X|L1], L2) :-
   X < M,!,
   algorithman_quicksort_pivot(M, R, L1, L2).
algorithman_quicksort_pivot(M, [X|R], L1, [X|L2]) :-
   X >= M,
   algorithman_quicksort_pivot(M, R, L1, L2).
   
algorithman_quicksort([],[]).
algorithman_quicksort([Median|R], Sorted) :-
   algorithman_quicksort_pivot(Median, R, Left, Right),
   algorithman_quicksort(Left, LeftSorted),
   algorithman_quicksort(Right, RightSorted),
   algorithman_append(LeftSorted, [Median|RightSorted], Sorted).

